from setuptools import setup
from setuptools_rust import RustBin, Strip

setup(
    package_dir={"ftl_extract": "src/ftl_extract"},
    rust_extensions=[
        RustBin(
            "ftl",
            path="src/cli/Cargo.toml",
            strip=Strip.All,
            debug=False,
        )
    ],
)
